#include "dommodel.h"
#include <QtXml>
#include <QtWidgets>
#include <QAbstractItemModel>
#include <iostream>

DomModel::DomModel(QWidget* p_parent): QTreeView(p_parent)    {
    this->setEditTriggers(QTreeView::NoEditTriggers);
    model = new QStandardItemModel(this);
    model->setColumnCount(4);
    model->setHorizontalHeaderLabels(QStringList() << "Поле" << "Организация" << "Документы" << "Значение");
    p_item = model->invisibleRootItem();
    customMenu = new QMenu(this);

    customMenu->addAction("Сделать активным");
    customMenu->addAction("Закрыть");

    this->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(this, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(context_menu(QPoint)));
    connect(customMenu, SIGNAL(triggered(QAction*)), SLOT(context_menu_reader(QAction*)));
}

void DomModel::load()
{
    QString openXmlFileDirectory = QFileDialog::getOpenFileName();
    if (f_directoryList.contains(openXmlFileDirectory))
        return;

    f_directoryList << openXmlFileDirectory;


    QDomDocument domDoc;
    f_currentFile = new QFile(openXmlFileDirectory);
    QStringList list = openXmlFileDirectory.split("/");
    QString fName = list.last();

    if (f_currentFile->open(QIODevice::ReadOnly))
    {
        if (domDoc.setContent(f_currentFile))
        {
            QDomElement domElem = domDoc.documentElement();
            if(domElem.isNull())
                QMessageBox::warning(this, "Внимание!","Файл пуст!");
            QStandardItem *item1 = new QStandardItem(fName);
            rootList << item1;
            p_item->appendRow(item1);

            traverseNode(domElem, item1);

            set_root(model->indexFromItem(item1));
        }
        else
        {
            QMessageBox::warning(this, "Внимание!","Файл поврежден!");
        }
        f_currentFile->close();
    }
    else
    {
        QMessageBox::warning(this, "Внимание!","Не получилось открыть файл!");
    }
    setModel(model);

}
void DomModel::load(const QString& f_directory)
{
    f_directoryList << f_directory;

    QDomDocument domDoc;
    f_currentFile = new QFile(f_directory);
    QStringList list = f_directory.split("/");
    QString fName = list.last();

    if (f_currentFile->open(QIODevice::ReadOnly))
    {
        if (domDoc.setContent(f_currentFile))
        {
            QDomElement domElem = domDoc.documentElement();
            QStandardItem *item2 = new QStandardItem(fName);
            rootList << item2;
            p_item->appendRow(item2);

            traverseNode(domElem, item2);

            set_root(model->indexFromItem(item2));
        }
        f_currentFile->close();
    }

    setModel(model);
}

void DomModel::simpleLoad(const QString &f_directory)
{
    f_directoryList << f_directory;

    QDomDocument domDoc;
    f_currentFile = new QFile(f_directory);
    QStringList list = f_directory.split("/");
    QString fName = list.last();

    if (f_currentFile->open(QIODevice::ReadOnly))
    {
        if (domDoc.setContent(f_currentFile))
        {
            QDomElement domElem = domDoc.documentElement();
            QStandardItem *item3 = new QStandardItem(fName);
            rootList << item3;
            if(p_item->rowCount() != 0)
                p_item->removeRow(0);
            p_item->appendRow(item3);

            simpleTraverseNode(domElem, item3);

        }
        f_currentFile->close();
    }
    setModel(model);
}

void DomModel::traverseNode(const QDomNode& p_dNode, QStandardItem* p_item_ptr)
{
    QDomNode domNode = p_dNode.firstChild();
    while (!domNode.isNull())
    {
        if (domNode.isElement())
        {
            QDomElement domElement = domNode.toElement();

            if (!domElement.isNull())
            {

                if (domElement.tagName() == "array")
                {
                    QStandardItem *item_arr = new QStandardItem();
                    QStandardItem *item_text = new QStandardItem();
                    item = item_text;
                    p_item_ptr->appendRow({item_arr, item_text});
                    traverseNode(domNode, item_arr);

                }
                else if (domElement.tagName() == "geoData")
                {
                    QStandardItem *item_gD = new QStandardItem(domElement.tagName());
                    p_item_ptr->appendRow(item_gD);
                    traverseNode(domNode, item_gD);
                }
                else if (domElement.tagName() == "Name")
                {
                    item->setText(domElement.text());
                    traverseNode(domNode, p_item_ptr);
                }
                else
                {
                    QStandardItem *item_tag = new QStandardItem(domElement.tagName());
                    QStandardItem *item_text = new QStandardItem(domElement.text());
                    p_item_ptr->appendRow({nullptr, nullptr, item_tag, item_text});
                    traverseNode(domNode, p_item_ptr);
                }


            }
        }

        domNode = domNode.nextSibling();

    }
}

void DomModel::simpleTraverseNode(const QDomNode &p_dNode, QStandardItem *p_item_ptr)
{
    QDomNode domNode = p_dNode.firstChild();
    while (!domNode.isNull())
    {
        if (domNode.isElement())
        {
            QDomElement domElement = domNode.toElement();

            if (!domElement.isNull())
            {
                QStandardItem *item_tag = new QStandardItem(domElement.tagName());
                QStandardItem *item_text = new QStandardItem(domElement.text());
                p_item_ptr->appendRow({item_tag, item_text});
                simpleTraverseNode(domNode, p_item_ptr);
            }
        }

        domNode = domNode.nextSibling();

    }

}



void DomModel::close()
{
    if (rootList.isEmpty()) return;
    for(auto root : rootList)
    {
        drop_single(model->indexFromItem(root));
    }
    rootList.clear();
    f_directoryList.clear();
}

void DomModel::drop_single(QModelIndex p_idx)
{
    model->invisibleRootItem()->removeRow(p_idx.row());
    f_directoryList.removeAt(p_idx.row());
    rootList.removeAt(p_idx.row());
}

QList<QStandardItem*>* DomModel::get_root()
{
    return &rootList;
}

void DomModel::set_root()
{
    QString root = qobject_cast <QAction*>(sender())->text();
    auto p_iter = rootList.begin();
    int counter = 0;
    while (p_iter != rootList.end())
    {
        if(p_iter.i->t()->text() == root)
        {
            scrollTo(model->indexFromItem(p_iter.i->t()));
            p_iter.i->t()->setFont(QFont("Times,12,-1,5,50,0,0,0,0,0", 9, QFont::Bold));
            f_currentFile = new QFile(f_directoryList.at(counter));
            emit sigSendFile(f_currentFile);
        }
        else
        {
            p_iter.i->t()->setFont(QFont ("Times,12,-1,5,50,0,0,0,0,0", 9));
        }
        counter++;
        p_iter++;
    }
}

void DomModel::set_root(QModelIndex p_idx)
{
    size_t pos = 0;
    for (auto root : rootList)
    {
        if (p_idx == model->indexFromItem(root))
        {
            scrollTo(model->indexFromItem(root));
            root->setFont(QFont("Times,12,-1,5,50,0,0,0,0,0", 9, QFont::Bold));
            f_currentFile = new QFile(f_directoryList.at(pos));
            emit sigSendFile(f_currentFile);
        }
        else
        {
            root->setFont(QFont ("Times,12,-1,5,50,0,0,0,0,0", 9));
        }
        pos++;
    }
}


void DomModel::context_menu(QPoint pos)
{
    bool isRoot = false;
    for (auto elem : rootList)
    {
        if(this->indexAt(pos) == model->indexFromItem(elem))
        isRoot = true;
    }

    if (isRoot)
    {
        index = this->indexAt(pos);
        customMenu->exec(mapToGlobal(pos));
    }

}

void DomModel::context_menu_reader(QAction* p_act)
{
    if (customMenu->actions().indexOf(p_act) == 0)
    {
        set_root(index);
    }
    else
    {
        drop_single(index);
    }
}
